///File : ItemSectionDBBase.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "ItemSectionDBBase.hpp"
//------------------------------------------------------------------------------
ItemSectionDBBase::ItemSectionDBBase(const std::string &sectionDBPath, const std::string &itemDBPath )
{
    InitSectionItemDB(sectionDBPath, itemDBPath );
}
//------------------------------------------------------------------------------
void ItemSectionDBBase::InitSectionItemDB(const std::string &sectionDBPath, const std::string &itemDBPath)
{
    try
    {
        LoadSectionDB(sectionDBPath);
        LoadItemDB(itemDBPath);
        m_isOk = true;
    }
    catch(const std::exception& e) //Rattrape toutes les exceptions
    {
        std::cerr << e.what() << std::endl;
        m_isOk = false;
    }

    m_isSectionsModified = false;
    m_isItemsModified = false;
}
//------------------------------------------------------------------------------
void ItemSectionDBBase::SaveSectionDB(const std::string &filename)
{
    ticpp::Document myDoc(filename);

    ticpp::Declaration myDeclaration("1.0","UTF-8","no");

    ticpp::Element Version("version");
    Version.SetText("1");

    ticpp::Element Sections("sections");

    for(int i = 0; i < m_sectionDB.size(); i++)
    {
        ticpp::Element *Section = new ticpp::Element("section");

        Section->SetAttribute("name", m_sectionDB[i]);

        Sections.InsertEndChild(*Section);
    }

    ticpp::Element Base("sectionDB");
    Base.InsertEndChild(Version);
    Base.InsertEndChild(Sections);

    myDoc.InsertEndChild(myDeclaration);
    myDoc.InsertEndChild(Base);

    myDoc.SaveFile();

    m_isSectionsModified = false;
}
//------------------------------------------------------------------------------

void ItemSectionDBBase::SaveItemDB(const std::string &filename)
{
    ticpp::Document myDoc(filename);

    ticpp::Declaration myDeclaration("1.0","UTF-8","no");

    ticpp::Element Version("version");
    Version.SetText("1");

    ticpp::Element Items("items");

    std::vector<std::string> itemsRefList = GetAllItemsList();
    for(int i = 0; i < itemsRefList.size(); i++)
    {
        ticpp::Element *Item = new ticpp::Element("item");

        Item->SetAttribute("ref", itemsRefList[i]);
        Item->SetAttribute("name", GetItemName(itemsRefList[i]));
        Item->SetAttribute("priceHT", GetItemPriceHT(itemsRefList[i]));
        Item->SetAttribute("taxe", GetItemTaxe(itemsRefList[i]));
        Item->SetAttribute("ecoPart", GetItemEcoPart(itemsRefList[i]));

        Items.InsertEndChild(*Item);
    }

    ticpp::Element Base("itemDB");
    Base.InsertEndChild(Version);
    Base.InsertEndChild(Items);

    myDoc.InsertEndChild(myDeclaration);
    myDoc.InsertEndChild(Base);

    myDoc.SaveFile();

    m_isItemsModified = false;
}

//------------------------------------------------------------------------------
void ItemSectionDBBase::LoadSectionDB(const std::string &filename)
{
    m_sectionDB.clear();

    ticpp::Document myDoc(filename);
    myDoc.LoadFile();

    ticpp::Element sectionDBBase = *myDoc.FirstChild("sectionDB")->ToElement();
    if(sectionDBBase.FirstChild("version")->ToElement()->GetText() == "1")
    {
        ticpp::Element Sections = *sectionDBBase.FirstChild("sections")->ToElement();

        ticpp::Element *SectionElement = Sections.FirstChildElement("section", false);
        while(SectionElement != NULL)
        {

            m_sectionDB.push_back(SectionElement->GetAttribute("name"));

            SectionElement = SectionElement->NextSiblingElement("section", false);
        }
    }
    else
    {
        throw ticpp::Exception("Bad file version !");
    }

    m_isSectionsModified = false;
}
//------------------------------------------------------------------------------

void ItemSectionDBBase::LoadItemDB(const std::string &filename)
{
    m_itemDB.clear();

    ticpp::Document myDoc(filename);
    myDoc.LoadFile();

    ticpp::Element itemDBBase = *myDoc.FirstChild("itemDB")->ToElement();
    if(itemDBBase.FirstChild("version")->ToElement()->GetText() == "1")
    {
        ticpp::Element Items = *itemDBBase.FirstChild("items")->ToElement();

        ticpp::Element *ItemElement = Items.FirstChildElement("item", false);
        while(ItemElement != NULL)
        {
            itemRef temp;
            temp.name = ItemElement->GetAttribute("name");
            temp.priceHT = stringToNb<double>(ItemElement->GetAttribute("priceHT"));
            temp.taxe = stringToNb<double>(ItemElement->GetAttribute("taxe"));
            temp.ecoPart = stringToNb<double>(ItemElement->GetAttribute("ecoPart"));

            m_itemDB[ItemElement->GetAttribute("ref")] = temp;

            ItemElement = ItemElement->NextSiblingElement("item", false);
        }
    }
    else
    {
        throw ticpp::Exception("Bad file version !");
    }

    m_isItemsModified = false;
}
//------------------------------------------------------------------------------

bool ItemSectionDBBase::SectionExists(const std::string &sectionID) const
{
    return(GetSectionPosInDB(sectionID) != -1);
}
//------------------------------------------------------------------------------
int ItemSectionDBBase::GetSectionPosInDB(const std::string &sectionID) const
{
    for(int i = 0; i < m_sectionDB.size(); i++)
    {
        if(sectionID == m_sectionDB[i])
        {
            return i;
        }
    }

    return -1;
}
//------------------------------------------------------------------------------
std::vector<std::string> ItemSectionDBBase::GetAllItemsList() const
{
    std::vector<std::string> tempList;
    std::map<std::string, itemRef>::const_reverse_iterator rit;
    for ( rit = m_itemDB.rbegin() ; rit != m_itemDB.rend(); rit++ )
    {
        tempList.push_back(rit->first);
    }

    return tempList;
}
//------------------------------------------------------------------------------
int ItemSectionDBBase::GetAllItemsCount() const
{
    return m_itemDB.size();
}
//------------------------------------------------------------------------------
std::vector<std::string> ItemSectionDBBase::GetAllSectionsList() const
{
    return m_sectionDB;
}
//------------------------------------------------------------------------------
int ItemSectionDBBase::GetAllSectionsCount() const
{
    return m_sectionDB.size();
}
//------------------------------------------------------------------------------

double ItemSectionDBBase::GetItemPriceHT(const std::string &itemID) const
{
    if(m_itemDB.find(itemID) != m_itemDB.end())
    {
        return m_itemDB.at(itemID).priceHT;
    }
    else
    {
        throw ticpp::Exception("No item with this id !");
    }
}
//------------------------------------------------------------------------------
double ItemSectionDBBase::GetItemTaxe(const std::string &itemID) const
{
    if(m_itemDB.find(itemID) != m_itemDB.end())
    {
        return m_itemDB.at(itemID).taxe;
    }
    else
    {
        throw ticpp::Exception("No item with this id !");
    }
}
//------------------------------------------------------------------------------
double ItemSectionDBBase::GetItemEcoPart(const std::string &itemID) const
{
    if(m_itemDB.find(itemID) != m_itemDB.end())
    {
        return m_itemDB.at(itemID).ecoPart;
    }
    else
    {
        throw ticpp::Exception("No item with this id !");
    }
}
//------------------------------------------------------------------------------

std::string ItemSectionDBBase::GetItemName(const std::string &itemID) const
{
    if(m_itemDB.find(itemID) != m_itemDB.end())
    {
        return m_itemDB.at(itemID).name;
    }
    else
    {
        throw ticpp::Exception("No item with this id !");
    }
}

//------------------------------------------------------------------------------
bool ItemSectionDBBase::ItemExists(const std::string &itemID) const
{
    if(m_itemDB.find(itemID) != m_itemDB.end())
    {
        return true;
    }
    else
    {
        return false;
    }
}

//------------------------------------------------------------------------------
bool ItemSectionDBBase::IsOk() const
{
    return m_isOk;
}
//------------------------------------------------------------------------------

bool ItemSectionDBBase::IsModified() const
{
    if(m_isSectionsModified == true || m_isItemsModified == true)
    {
        return true;
    }
    else
    {
        return false;
    }
}
