///File : ItemSectionDBManager.cpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "ItemSectionDBManager.hpp"
//------------------------------------------------------------------------------
ItemSectionDBManager::ItemSectionDBManager(const std::string &sectionDBPath, const std::string &itemDBPath ) : ItemSectionDBBase(sectionDBPath, itemDBPath )
{
    m_SectionDBPath = sectionDBPath;
    m_ItemDBPath = itemDBPath;

    // Backup
    BackupSaveCurrent(BACKUP_NEW);
}
//------------------------------------------------------------------------------
bool ItemSectionDBManager::ItemDBAdd(const std::string &itemID)
{
    if(!ItemExists(itemID))
    {
        m_itemDB[itemID].priceHT = 0;
        m_itemDB[itemID].ecoPart = 0;
        m_itemDB[itemID].taxe = 19.6;
        m_itemDB[itemID].name = "";
        m_isItemsModified = true;
        BackupSaveCurrent(BACKUP_ITEM_DB_ADDED, itemID);
        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------

void ItemSectionDBManager::SectionDBSetName(const std::string &sectionID, const std::string &name)
{
    int pos = 0;
    pos = GetSectionPosInDB(sectionID);

    if(pos != -1)
    {
        m_sectionDB[pos] = name;
        m_isItemsModified = true;
        BackupSaveCurrent(BACKUP_SECTION_DB_MODIFIED, sectionID);
    }
}

//------------------------------------------------------------------------------
bool ItemSectionDBManager::ItemDBSetRef(const std::string &itemID, const std::string &newItemID)
{
    if(ItemExists(itemID) && !ItemExists(newItemID)) // Si l'Id courant existe et que le noàuveau non...
    {
        m_itemDB[newItemID] = m_itemDB[itemID];
        m_itemDB.erase(itemID); // On supprime l'ancien
        m_isItemsModified = true;
        BackupSaveCurrent(BACKUP_ITEM_DB_MODIFIED, itemID);
        return true;
    }
    else
    {
        return false;
    }
}
//------------------------------------------------------------------------------

void ItemSectionDBManager::ItemDBSetName(const std::string &itemID, const std::string &name)
{
    if(ItemExists(itemID))
    {
        m_itemDB[itemID].name = name;
        m_isItemsModified = true;
        BackupSaveCurrent(BACKUP_ITEM_DB_MODIFIED, itemID);
    }
}
//------------------------------------------------------------------------------

void ItemSectionDBManager::ItemDBSetPriceHT(const std::string &itemID, const double &priceHT)
{
    if(ItemExists(itemID))
    {
        m_itemDB[itemID].priceHT = priceHT;
        m_isItemsModified = true;
        BackupSaveCurrent(BACKUP_ITEM_DB_MODIFIED, itemID);
    }
}
//------------------------------------------------------------------------------
void ItemSectionDBManager::ItemDBSetTaxe(const std::string &itemID, const double &taxe)
{
    if(ItemExists(itemID))
    {
        m_itemDB[itemID].taxe = taxe;
        m_isItemsModified = true;
        BackupSaveCurrent(BACKUP_ITEM_DB_MODIFIED, itemID);
    }
}
//------------------------------------------------------------------------------
void ItemSectionDBManager::ItemDBSetEcoPart(const std::string &itemID, const double &ecoPart)
{
    if(ItemExists(itemID))
    {
        m_itemDB[itemID].ecoPart = ecoPart;
        m_isItemsModified = true;
        BackupSaveCurrent(BACKUP_ITEM_DB_MODIFIED, itemID);
    }
}
//------------------------------------------------------------------------------
void ItemSectionDBManager::ItemDBRemove(const std::string &itemID)
{
    m_itemDB.erase(itemID);
    m_isItemsModified = true;
    BackupSaveCurrent(BACKUP_ITEM_DB_REMOVED, itemID);
}
//------------------------------------------------------------------------------

void ItemSectionDBManager::SectionDBAdd(const std::string &sectionID)
{
    int pos = GetSectionPosInDB(sectionID);

    if(pos == -1) // New
    {
        m_sectionDB.push_back(sectionID);
        m_isSectionsModified = true;
        BackupSaveCurrent(BACKUP_SECTION_DB_ADDED, sectionID);
    }
}
//------------------------------------------------------------------------------

void ItemSectionDBManager::SectionDBRemove(const std::string &sectionID)
{
    int pos = GetSectionPosInDB(sectionID);
    if(pos != -1)
    {
        m_sectionDB.erase(m_sectionDB.begin() + pos);
        m_isSectionsModified = true;
        BackupSaveCurrent(BACKUP_SECTION_DB_REMOVED, sectionID);
    }
}
//------------------------------------------------------------------------------

void ItemSectionDBManager::SaveDBs()
{
    SaveSectionDB(m_SectionDBPath);
    SaveItemDB(m_ItemDBPath);

    m_backup[m_backupPos].m_isItemsModified = false; // Le backup est lui aussi sauvergardé (car c'est le même que les données actueles).
    m_backup[m_backupPos].m_isSectionsModified = false; // Le backup est lui aussi sauvergardé (car c'est le même que les données actueles).
}
//------------------------------------------------------------------------------
// Backup
bool ItemSectionDBManager::BackupCanGoTo(const int &pos) const
{
    // Si il y a au moins un backup et que la position Test appartient au backup.
    int testPos = m_backupPos + pos;
    return(m_backup.size() > 1 && testPos >= 0 && testPos <= m_backup.size() - 1);
}
//------------------------------------------------------------------------------
void ItemSectionDBManager::BackupSaveCurrent(const int &type, const std::string &infoStr)
{
    if(m_backup.size() > 1 && m_backupPos != m_backup.size() - 1) // Si il y a des backup après la pos actuelle
    {
        // ==> Il faut les supprimer (on commence une nouvelle version)
        m_backup.erase(m_backup.begin() + m_backupPos + 1,  m_backup.end());
    }

    m_backup.resize(m_backup.size() + 1); // Ajout d'un enregistrement.

    // Data :
    int pos = m_backup.size() - 1;
    m_backup[pos].m_sectionDB = m_sectionDB;
    m_backup[pos].m_itemDB = m_itemDB;
    m_backup[pos].m_isOk = m_isOk;
    m_backup[pos].m_isSectionsModified = m_isSectionsModified;
    m_backup[pos].m_isItemsModified = m_isItemsModified;

    m_backup[pos].type = type;
    m_backup[pos].infoStr = infoStr;

    // Epuration...
    if(m_backup.size() > BACKUP_NB) // Si trop de backup...
    {
        m_backup.erase(m_backup.begin()); // On supprime le premier (le plus ancien).
    }

    // Current backup ==> Le dernier.
    m_backupPos = m_backup.size() - 1;
}
//------------------------------------------------------------------------------
void ItemSectionDBManager::BackupLoad(const int &pos)
{
    int newPos = m_backupPos + pos;
    if(newPos >= 0 && newPos < m_backup.size()) // Si nouvelle pos dans le tableau.
    {
        m_backupPos = newPos;
        m_sectionDB = m_backup[m_backupPos].m_sectionDB;
        m_itemDB = m_backup[m_backupPos].m_itemDB;
        m_isOk = m_backup[m_backupPos].m_isOk;
        m_isSectionsModified = m_backup[m_backupPos].m_isSectionsModified;
        m_isItemsModified = m_backup[m_backupPos].m_isItemsModified;
    }
    else
    {
        throw ticpp::Exception("Backup out of range !");
    }
}
//------------------------------------------------------------------------------
int ItemSectionDBManager::BackupGetType(const int &pos) const
{
    int newPos = m_backupPos + pos;
    if(newPos >= 0 && newPos < m_backup.size()) // Si nouvelle pos dans le tableau.
    {
        return m_backup[newPos].type;
    }
    else
    {
        throw ticpp::Exception("Backup out of range !");
    }
}
//------------------------------------------------------------------------------
std::string ItemSectionDBManager::BackupGetInfoStr(const int &pos) const
{
    int newPos = m_backupPos + pos;
    if(newPos >= 0 && newPos < m_backup.size()) // Si nouvelle pos dans le tableau.
    {
        return m_backup[newPos].infoStr;
    }
    else
    {
        throw ticpp::Exception("Backup out of range !");
    }
}
